// DE_method.h: interface for the CDE_method class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_DE_METHOD_H__C03C060E_3664_433D_A986_EB3690699859__INCLUDED_)
#define AFX_DE_METHOD_H__C03C060E_3664_433D_A986_EB3690699859__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000


#include "global.h"

#include "./EACommonClass/Individual.h"
#include "./EACommonClass/Rand.h"
#include "./EACommonClass/ProblemDef.h"


#include <direct.h>				// include the mkdir() for folder creation

#define out_internal	30		// Ƶ

/* Ӳ̣ԲҪһֽһֽڸ */
#define SIZEOFBUFFER	256*1024L /* СĬΪ256KB */


class CDE_method  
{
public:
	CDE_method();
	virtual ~CDE_method();

	/* ------------------------class methods------------------------ */
public:
	// public methods
	/* initialize the decision variables of different test problems */
	void init_variables();							// initialize the decision variables
	void init_normal_variables();					// initialize the normal functions' variables
	
	/* initialize the population */
	void init_pop();								// initialize the population
	void init_pop_random();							// initialize the population randomly

	/* evaluate the individual and population */
	void evaluate_ind(CIndividual &);				// evaluate the individual
	void evaluate_normal_ind(CIndividual &);		// evaluate the individual of normal functions

	void evaluate_pop(CIndividual *, int);			// evaluate the population

	/* execute the algorithms */
	void Run_Optimizer(int, double, int, int);		// execute the optimizer

	/* compare the two individuals */
	int  compare_ind(CIndividual *, CIndividual *);	// compare the two individuals
	void find_best_index(CIndividual *, int);		// find the index of the best individual in the population

	void restart_pop(CIndividual *, int);			// restart the population
	void sort_pop(CIndividual *, int);				// sort the population w.r.t fitness with ascend order

	/* some complementary methods */
	void random_index(int *array_index, int all_size, int size);

	void shell_sort_pop(CIndividual* ,int *, int);		// shell sort routine in ascending order
	void shell_sort_array(double *, int *, int);
	void shell_sort_array_t(tFitness *, int *, int);

	inline  int round(double x) { return int(x>0.0?(x+0.5):(x-0.5)); } // round function

	double median(double *array, int size);			// get the median value of the specific array
	double mean_std(double *array, int size, double &stdv);	// get the mean and standard values of the specific array
	tFitness mean_std_t(tFitness *array, int size, tFitness &stdv);
	void   statistic_pop_fitness(CIndividual *pop, int size);

	void display_result(long int);						// display the results in the screen
	void report_result(long int, ofstream &);			// report the results in the files
	void report_diversity(long int, ofstream &);			// report the diversity in the files
	void report_parameter(long int gen, ofstream &file);

	/************************************************************************/
	/* Methods and attributes for batch execution (ִг)      */
	/************************************************************************/
public:
	void	create_folders(int method_ii, int strategy_ii);		// create the folders
	void	copy_files(char *source, char *target);				// copy files from the source directory to the target directory
	long	filesize(FILE *stream);
	int		copyfile(const char* src,const char* dest);

	/* ------------------------class members------------------------ */	
public:
	// public attributes
	CIndividual *parent_pop;				// the main population
	CIndividual *child_pop;					// the child population
	CIndividual *child_pop1;				// the child population used in BBO_DE
	CIndividual best_individual;			// the current best individual

	CIndividual	mutant_vector;				// save the mutant vector
	CIndividual new_child;					// the new child generated by the genetic operator
	
	int pop_size;							// size of the main population
	int best_index;							// index of the current best individual
	int worst_index;

	int flag_precision;						// stop criterion

	long int max_NFFEs;						// maximal number of fitness function evaluations
	long int max_iterations;				// maximal number of iterations
	long int evaluations;					// current number of fitness function evaluations
	long int gen;							// current generation counter
	int MINIMIZE;

	double *upper_bounds;					// upper bounds of the decision variables
	double *lower_bounds;					// lower bounds of the decision variables
	double known_optimal;					// known optima of the problems

	double pop_diversity;
	double max_interval;

	/* some other class objectives */
	double rnd_seed;
	CRand  m_rnd;

	int func_flag;							// flag of the function optimized
	CProblemDef	m_func;						// normal test functions

	int method_flag;

	int	*sort_index;						// only for sorting the population

	/************************************************************************/
	/* Methods and attributes for JADE methods                              */
	/************************************************************************/
public:	
	int		m_strategy;
	double	m_F, m_CR;

	double	JADE_mu_CR;
	double	JADE_mu_FF;
	double	JADE_cr_std, JADE_ff_std;
	double	JADE_c;
	double	JADE_S_FF[population_size];				// save the successful F
	double	JADE_S_CR[population_size];				// save the successful CR

	double		JADE_p;								// the control parameter of p
	int         archive_size;						// the archive size of the archive
	CIndividual archive_pop[population_size];		// the archive of JAVE

	void	run_JADE_method();						// DE only with JADE parameter adaptation technique

	/************************************************************************/
	/* Methods and attributes for transfer adaptive DE (TRADE)              */
	/************************************************************************/
public:
	static double	save_JADE_mu_CR;
	static double	save_JADE_mu_FF;

	double	JADE_DB_data[1000][2];
	int		JADE_DB_num;

	int		the_same_best_fitness;

	int		load_data_from_DB_JADE();					// load saved data from database
	void	process_data_JADE();

	void	run_transfer_JADE();

};

#endif // !defined(AFX_DE_METHOD_H__C03C060E_3664_433D_A986_EB3690699859__INCLUDED_)
